@@ -5,3 +5,6 @@ from .set_of_files import set_of_files
 from .keyword_lookup import keyword_lookup
 from .detector_lookup import detector_lookup
 from .create_dispersion_map import create_dispersion_map
+from .getpackagepath import getpackagepath
+from .detect_continuum import detect_continuum
+from . import polynomials
