@@ -31,6 +31,7 @@ import math
 from photutils.utils import NoDetectionsWarning
 import warnings
 from astropy.visualization import hist
+from soxspipe.commonutils.polynomials import chebyshev_order_wavelength_polynomials
 
 
 class create_dispersion_map(object):
@@ -131,8 +132,8 @@ class create_dispersion_map(object):
         observed_x = [x for x in observed_x if x != None]
         observed_y = [y for y in observed_y if y != None]
 
-        order_deg = self.settings["soxs-disp-solution"]["order_deg"]
-        wavelength_deg = self.settings["soxs-disp-solution"]["wavelength_deg"]
+        order_deg = self.settings["soxs-disp-solution"]["order-deg"]
+        wavelength_deg = self.settings["soxs-disp-solution"]["wavelength-deg"]
 
         # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
         popt_x, popt_y = self.fit_polynomials(
@@ -147,7 +148,6 @@ class create_dispersion_map(object):
         # WRITE THE MAP TO FILE
         mapPath = self.write_map_to_file(
             popt_x, popt_y, order_deg, wavelength_deg)
-        print(f"Single pinhole first guess dispersion map saved to: {mapPath}")
 
         self.log.debug('completed the ``get`` method')
         return mapPath
@@ -256,38 +256,6 @@ class create_dispersion_map(object):
         self.log.debug('completed the ``detect_pinhole_arc_line`` method')
         return observed_x, observed_y
 
-    def chebyshev_polynomials_single(self, order_wave, *coeff):
-        """*the chebyshev polynomial fits for the single pinhole frames; to be iteratively fitted to minimise errors*
-
-        **Key Arguments:**
-            - ``order_wave`` -- a tuple of the order and wavelength arrays
-            - ``*coeff`` -- a list of the initail coefficients
-
-        **Return:**
-            - ``lhsVals`` -- the left-hand-side vals of the fitted polynomials
-        """
-        self.log.debug('starting the ``chebyshev_polynomials_single`` method')
-
-        # UNPACK TUPLE INPUT
-        orders = order_wave[0]
-        wavelengths = order_wave[1]
-        lhsVals = []
-        order_deg = self.settings["soxs-disp-solution"]["order_deg"]
-        wavelength_deg = self.settings["soxs-disp-solution"]["wavelength_deg"]
-
-        # POLYNOMIALS SUMS
-        for order, wave in zip(orders, wavelengths):
-            n_coeff = 0
-            val = 0
-            for i in range(0, order_deg + 1):
-                for j in range(0, wavelength_deg + 1):
-                    val += coeff[n_coeff] * \
-                        math.pow(order, i) * math.pow(wave, j)
-                    n_coeff += 1
-            lhsVals.append(val)
-        self.log.debug('completed the ``chebyshev_polynomials_single`` method')
-        return lhsVals
-
     def write_map_to_file(
             self,
             xcoeff,
@@ -314,8 +282,8 @@ class create_dispersion_map(object):
         # SORT X COEFFICIENT OUTPUT TO WRITE TO FILE
         coeff_dict_x = {}
         coeff_dict_x["axis"] = "x"
-        coeff_dict_x["order_deg"] = order_deg
-        coeff_dict_x["wavelength_deg"] = wavelength_deg
+        coeff_dict_x["order-deg"] = order_deg
+        coeff_dict_x["wavelength-deg"] = wavelength_deg
         n_coeff = 0
         for i in range(0, order_deg + 1):
             for j in range(0, wavelength_deg + 1):
@@ -325,8 +293,8 @@ class create_dispersion_map(object):
         # SORT Y COEFFICIENT OUTPUT TO WRITE TO FILE
         coeff_dict_y = {}
         coeff_dict_y["axis"] = "y"
-        coeff_dict_y["order_deg"] = order_deg
-        coeff_dict_y["wavelength_deg"] = wavelength_deg
+        coeff_dict_y["order-deg"] = order_deg
+        coeff_dict_y["wavelength-deg"] = wavelength_deg
         n_coeff = 0
         for i in range(0, order_deg + 1):
             for j in range(0, wavelength_deg + 1):
@@ -356,8 +324,7 @@ class create_dispersion_map(object):
             ycoeff,
             order_deg,
             wavelength_deg,
-            slit_deg=False,
-            plot=False):
+            slit_deg=False):
         """*calculate residuals of the polynomial fits against the observed line postions*
 
         **Key Arguments:**
@@ -370,10 +337,12 @@ class create_dispersion_map(object):
             - ``order_deg`` -- degree of the order fitting
             - ``wavelength_deg`` -- degree of wavelength fitting
             - ``slit_deg`` -- degree of the slit fitting (False for single pinhole)
-            - ``plot`` -- write out a plot to file. Default False.
 
         **Return:**
             - ``residuals`` -- combined x-y residuals
+            - ``mean`` -- the mean of the combine residuals
+            - ``std`` -- the stdev of the combine residuals
+            - ``median`` -- the median of the combine residuals
         """
         self.log.debug('starting the ``calculate_residuals`` method')
 
@@ -382,12 +351,15 @@ class create_dispersion_map(object):
         # POLY FUNCTION NEEDS A TUPLE AS INPUT
         order_wave = (orders, wavelengths)
 
+        poly = chebyshev_order_wavelength_polynomials(
+            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg).poly
+
         # CALCULATE X & Y RESIDUALS BETWEEN OBSERVED LINE POSITIONS AND POLY
         # FITTED POSITIONS
-        residuals_x = np.asarray(self.chebyshev_polynomials_single(
-            order_wave, *xcoeff)) - np.asarray(observed_x)
-        residuals_y = np.asarray(self.chebyshev_polynomials_single(
-            order_wave, *ycoeff)) - np.asarray(observed_y)
+        xfit = poly(order_wave, *xcoeff)
+        yfit = poly(order_wave, *ycoeff)
+        residuals_x = np.asarray(xfit) - np.asarray(observed_x)
+        residuals_y = np.asarray(yfit) - np.asarray(observed_y)
 
         # CALCULATE COMBINED RESIDUALS AND STATS
         combined_res = np.sqrt(np.square(residuals_x) + np.square(residuals_y))
@@ -395,28 +367,8 @@ class create_dispersion_map(object):
         combined_res_std = np.std(combined_res)
         combined_res_median = np.median(combined_res)
 
-        # IF PLOT IS REQUIRED:
-        if plot:
-            fig, ax = plt.subplots(1, 2, figsize=(10, 4))
-            plt.subplots_adjust(top=0.85)
-            ax[0].scatter(residuals_x, residuals_y, alpha=0.4)
-            ax[0].set_xlabel('x')
-            ax[0].set_ylabel('y')
-
-            hist(combined_res, bins='scott', ax=ax[1], histtype='stepfilled',
-                 alpha=0.7, density=True)
-            ax[1].set_xlabel('xy residual')
-            subtitle = f"mean res: {combined_res_mean:2.3f} pix, res stdev: {combined_res_std:2.3f}"
-            fig.suptitle(
-                f"residuals of global dispersion solution fitting - single pinhole\n{subtitle}")
-
-            home = expanduser("~")
-            outDir = self.settings["intermediate-data-root"].replace("~", home)
-            filePath = f"{outDir}/single_pinhole_{arm}_disp_map_residuals.pdf"
-            plt.savefig(filePath)
-
         self.log.debug('completed the ``calculate_residuals`` method')
-        return combined_res
+        return combined_res, combined_res_mean, combined_res_std, combined_res_median, residuals_x, residuals_y, xfit, yfit
 
     def fit_polynomials(
             self,
@@ -444,20 +396,33 @@ class create_dispersion_map(object):
         """
         self.log.debug('starting the ``fit_polynomials`` method')
 
+        arm = self.arm
+
         clippedCount = 1
 
-        while clippedCount > 0:
+        poly = chebyshev_order_wavelength_polynomials(
+            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg).poly
+
+        clippingSigma = self.settings[
+            "soxs-disp-solution"]["poly-fitting-residual-clipping-sigma"]
+
+        clippingIterationLimit = self.settings[
+            "soxs-disp-solution"]["clipping-iteration-limit"]
+
+        iteration = 0
+        while clippedCount > 0 and iteration < clippingIterationLimit:
+            iteration += 1
             # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLYS
             # FIRST X
             coeff = np.ones((order_deg + 1) * (wavelength_deg + 1))
-            xcoeff, pcov_x = curve_fit(self.chebyshev_polynomials_single, xdata=(
+            xcoeff, pcov_x = curve_fit(poly, xdata=(
                 orders, wavelengths), ydata=observed_x, p0=coeff)
 
             # NOW Y
-            ycoeff, pcov_y = curve_fit(self.chebyshev_polynomials_single, xdata=(
+            ycoeff, pcov_y = curve_fit(poly, xdata=(
                 orders, wavelengths), ydata=observed_y, p0=coeff)
 
-            residuals = self.calculate_residuals(
+            residuals, mean_res, std_res, median_res, residuals_x, residuals_y, xfit, yfit = self.calculate_residuals(
                 observed_x=observed_x,
                 observed_y=observed_y,
                 wavelengths=wavelengths,
@@ -469,8 +434,6 @@ class create_dispersion_map(object):
                 slit_deg=False)
 
             # SIGMA-CLIP THE DATA
-            clippingSigma = self.settings[
-                "soxs-disp-solution"]["poly_fitting_residual_clipping_sigma"]
             masked_residuals = sigma_clip(
                 residuals, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
 
@@ -482,18 +445,66 @@ class create_dispersion_map(object):
             clippedCount = startCount - len(observed_x)
             print(f'{clippedCount} arc lines where clipped in this iteration of fitting a global dispersion map')
 
-        # PLOT THE RESIDUALS NOW CLIPPING IS COMPLETE
-        residuals = self.calculate_residuals(
-            observed_x=observed_x,
-            observed_y=observed_y,
-            wavelengths=wavelengths,
-            orders=orders,
-            xcoeff=xcoeff,
-            ycoeff=ycoeff,
-            order_deg=order_deg,
-            wavelength_deg=wavelength_deg,
-            slit_deg=False,
-            plot=True)
+        # a = plt.figure(figsize=(40, 15))
+        if arm == "UVB":
+            fig = plt.figure(figsize=(6, 13.5), constrained_layout=True)
+        else:
+            fig = plt.figure(figsize=(6, 11), constrained_layout=True)
+        gs = fig.add_gridspec(6, 4)
+
+        # CREATE THE GID OF AXES
+        toprow = fig.add_subplot(gs[0:2, :])
+        midrow = fig.add_subplot(gs[2:4, :])
+        bottomleft = fig.add_subplot(gs[4:, 0:2])
+        bottomright = fig.add_subplot(gs[4:, 2:])
+
+        # ROTATE THE IMAGE FOR BETTER LAYOUT
+        rotatedImg = np.rot90(self.pinholeFrame.data, 1)
+        toprow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
+        toprow.set_title(
+            "observed arc-line positions (post-clipping)", fontsize=10)
+        x = np.ones(observed_x.shape) * \
+            self.pinholeFrame.data.shape[1] - observed_x
+        toprow.scatter(observed_y, x, marker='x', c='red', s=4)
+        # toprow.set_yticklabels([])
+        # toprow.set_xticklabels([])
+        toprow.set_ylabel("x-axis", fontsize=8)
+        toprow.set_xlabel("y-axis", fontsize=8)
+        toprow.tick_params(axis='both', which='major', labelsize=9)
+
+        midrow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
+        midrow.set_title(
+            "global dispersion solution", fontsize=10)
+        xfit = np.ones(len(xfit)) * \
+            self.pinholeFrame.data.shape[1] - xfit
+        midrow.scatter(yfit, xfit, marker='x', c='blue', s=4)
+        # midrow.set_yticklabels([])
+        # midrow.set_xticklabels([])
+        midrow.set_ylabel("x-axis", fontsize=8)
+        midrow.set_xlabel("y-axis", fontsize=8)
+        midrow.tick_params(axis='both', which='major', labelsize=9)
+
+        # PLOT THE FINAL RESULTS:
+        plt.subplots_adjust(top=0.92)
+        bottomleft.scatter(residuals_x, residuals_y, alpha=0.4)
+        bottomleft.set_xlabel('x residual')
+        bottomleft.set_ylabel('y residual')
+        bottomleft.tick_params(axis='both', which='major', labelsize=9)
+
+        hist(residuals, bins='scott', ax=bottomright, histtype='stepfilled',
+             alpha=0.7, density=True)
+        bottomright.set_xlabel('xy residual')
+        bottomright.tick_params(axis='both', which='major', labelsize=9)
+        subtitle = f"mean res: {mean_res:2.2f} pix, res stdev: {std_res:2.2f}"
+        fig.suptitle(f"residuals of global dispersion solution fitting - single pinhole\n{subtitle}", fontsize=12)
+
+        # plt.show()
+        home = expanduser("~")
+        outDir = self.settings["intermediate-data-root"].replace("~", home)
+        filePath = f"{outDir}/single_pinhole_{arm}_disp_map_residuals.pdf"
+        plt.savefig(filePath)
+
+        print(f'\nThe dispersion maps fitted against the observed arc-line positions with a mean residual of {mean_res:2.2f} pixels (stdev = {std_res:2.2f} pixles)')
 
         self.log.debug('completed the ``fit_polynomials`` method')
         return xcoeff, ycoeff
