@@ -75,7 +75,7 @@ class soxs_disp_solution(_base_recipe_):
             settings=self.settings,
             inputFrames=self.inputFrames
         )
-        self.inputFrames = sof.get()
+        self.inputFrames, self.supplementaryInput = sof.get()
 
         # VERIFY THE FRAMES ARE THE ONES EXPECTED BY SOXS_disp_solution - NO MORE, NO LESS.
         # PRINT SUMMARY OF FILES.
@@ -157,9 +157,7 @@ class soxs_disp_solution(_base_recipe_):
         kw = self.kw
         dp = self.detectorParams
 
-        productPath = None
-
-        self.inputFrames.summary.pprint_all()
+        # self.inputFrames.summary.pprint_all()
 
         master_bias = False
         dark = False
@@ -177,7 +175,6 @@ class soxs_disp_solution(_base_recipe_):
 
         add_filters = {kw("DPR_TYPE"): 'LAMP,FMTCHK', kw("DPR_TECH"): 'IMAGE'}
         for i in self.inputFrames.files_filtered(include_path=True, **add_filters):
-            print(i)
             dark = CCDData.read(i, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                 hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
 
@@ -193,7 +190,7 @@ class soxs_disp_solution(_base_recipe_):
         if self.settings["save-intermediate-products"]:
             outDir = self.intermediateRootPath
             filePath = f"{outDir}/single_pinhole_{arm}_calibrated.fits"
-            print(f"Calibrated single pinhole frame: {filePath}")
+            print(f"\nCalibrated single pinhole frame: {filePath}\n")
             self._write(self.pinholeFrame, filePath, overwrite=True)
 
         from soxspipe.commonutils import create_dispersion_map
